--
-- CurrentZoneFix v2.0
-- by Legorol
-- email: legorol@cosmosui.org
--
-- Fixes a bug in the Blizzard API function SetMapToCurrentZone()
--
-- The bug:
-- -------
-- When you reload the User Interface in-game, the Blizzard API
-- function SetMapToCurrentZone() doesn't work correctly and only
-- zooms to the current continent rather than the current zone.
-- This behaviour persists until the first time you cross a
-- zone or area boundary.
-- This renders many AddOns unusable immediately after a User
-- Interface reload, for example MapNotes. This bug also means
-- that if you open the world map you are presented with the
-- continent rather than the zone map.
--
-- The Fix:
-- -------
-- This AddOn fixes this behaviour. The fix is smart, so
-- that once SetMapToCurrentZone() starts behaving normally,
-- the fix turns itself off to avoid unnecessary overhead.
--
-- For coders:
-- ----------
-- Simply copy the CurrentZoneFix_SetupFix() function and use
-- it in your AddOn if you like. It only needs to be called
-- once, best done in an OnLoad function, like it's done here.
-- It detects if the Fix is already present, so multiple AddOns
-- can safely include it. Please note that you need explicit
-- permission from the author to modify any of the code in
-- CurrentZoneFix_SetupFix()!
--

local function CurrentZoneFix_SetupFix()
	-- CurrentZoneFix v2.0 by Legorol
	-- email: legorol@cosmosui.org
	
	local versionID = 20;
	local id = CurrentZoneFix_FixInPlace;
	
	if ( id ) then
		id = tonumber(id);
		if ( id and id >= versionID ) then
			return;
		elseif ( not SetMapToCurrentZone(true) ) then
			DEFAULT_CHAT_FRAME:AddMessage("Warning! Obsolete version "..
				"of CurrentZoneFix function detected. The old "..
				"version is being loaded either as a standalone "..
				"AddOn, or as code embedded inside another AddOn. "..
				"You must update it to avoid bugs with the map!",1,0,0)
			return;
		end
	end
	
	local continent,zone,name
	local zoneID={};
	local orig_SetMapToCurrentZone;
	
	for continent in ipairs{GetMapContinents()} do
		for zone,name in ipairs{GetMapZones(continent)} do
			zoneID[name] = zone;
		end
	end
	
	orig_SetMapToCurrentZone = SetMapToCurrentZone;
	SetMapToCurrentZone = function(deactivate)
		if (deactivate) then
			SetMapToCurrentZone = orig_SetMapToCurrentZone;
			CurrentZoneFix_FixInPlace = nil;
			return true;
		end
		
		orig_SetMapToCurrentZone();
		if ( GetCurrentMapZone()==0 and GetCurrentMapContinent()>0 ) then
			SetMapZoom(GetCurrentMapContinent(),zoneID[GetRealZoneText()]);
		else
			SetMapToCurrentZone = orig_SetMapToCurrentZone;
			CurrentZoneFix_FixInPlace = "Deactivated "..versionID;
		end
	end
	
	CurrentZoneFix_FixInPlace = versionID;
end

function CurrentZoneFix_OnLoad()
	DEFAULT_CHAT_FRAME:AddMessage("CurrentZoneFix 2.0 by Legorol");
	CurrentZoneFix_SetupFix()
end
